
#include "unix.h"

typedef struct _Point {
    double x, y, z;
} Point;

typedef struct vertex {            /* surface vertex */
    Point position, normal;        /* position and surface normal */
} VERTEX;

typedef struct vertices {          /* list of vertices in polygonization */
    int count, max;                /* # vertices, max # allowed */
    VERTEX *ptr;                   /* dynamically allocated */
} VERTICES;

typedef unsigned int list[3];

double (*ptr1)();
double (*ptr2)();
double (*ptr3)();

int strbpos( s, c )
char *s, c;
{
    int i = -1;
    int l = strlen( s );
    for( i=l-1; i>=0; i-- )
        if( s[i] == c )
            return i;
    return i;
}

double function( x, y, z )
double x, y, z;
{
    return ptr1( x, y ) + ptr2( x, z ) + ptr3( y, z );
}


ImplicitFunction( xy, xz, yz, c, size, bounds, center )
char * xy, * xz, * yz, * c;
double size;
int bounds;
Point center;
{
    FILE	*fp, *fa, *fv;
    extern	char *filename;
    char	*aoff_filename;
    char	*vnorm_filename;
    char	*tmp_filename;
    int		i;
    char	*err, *polygonize();
    char	*tmpstr = (char *)malloc(80);

    Point	max, min;
    float	max_val, min_val, floor_min, ceili_max;
    float	scale;
    float	interval, interval_start;

    extern	int gntris;
    extern	int triangle();
    extern	VERTICES gvertices;
    extern	list gconnect[65536];

    extern double (*dfopen(char* expression))(double x, double y);

    matha2z();
    sprintf( tmpstr, "$(x,y)=%s", xy );
    ptr1 = dfopen( tmpstr );
    sprintf( tmpstr, "$(x,z)=%s", xz );
    ptr2 = dfopen( tmpstr );
    sprintf( tmpstr, "$(y,z)=%s + %s", yz, c );
    ptr3 = dfopen( tmpstr );

    gntris = 0;
    if ((err = polygonize(function, size, bounds, center.x, center.y, center.z, triangle)) != NULL) {
        fprintf(stderr, "%s\n", err);
        exit(1);
    }

    max.x = -65536.;    max.y = -65536.;    max.z = -65536.;
    min.x =  65536.;    min.y =  65536.;    min.z =  65536.;
    for( i = 0; i < gvertices.count; i++ ) {
        if( gvertices.ptr[i].position.x > max.x )
            max.x = gvertices.ptr[i].position.x;
        else if( gvertices.ptr[i].position.x < min.x )
            min.x = gvertices.ptr[i].position.x;
        if( gvertices.ptr[i].position.y > max.y )
            max.y = gvertices.ptr[i].position.y;
        else if( gvertices.ptr[i].position.y < min.y )
            min.y = gvertices.ptr[i].position.y;
        if( gvertices.ptr[i].position.z > max.z )
            max.z = gvertices.ptr[i].position.z;
        else if( gvertices.ptr[i].position.z < min.z )
            min.z = gvertices.ptr[i].position.z;
    }

    max_val = max( max( max.x, max.y ), max.z );
    min_val = min( min( min.x, min.y ), min.z );
    scale = max( max_val, min_val );

    for( i = 0; i < gvertices.count; i++ ) {
        gvertices.ptr[i].position.x /= scale;
        gvertices.ptr[i].position.y /= scale;
        gvertices.ptr[i].position.z /= scale;
    }

    fp = fopen( filename, "w" );
    fprintf(fp, "%d %d %d\n", gvertices.count, gntris, 3*gntris);
    for (i = 0; i < gvertices.count; i++) {
        fprintf(fp, "%f  %f  %f\n",
            gvertices.ptr[i].position.x,
            gvertices.ptr[i].position.y,
            gvertices.ptr[i].position.z);
        free( (char*)gvertices.ptr );
    }
    for (i = 0; i < gntris; i++)
        fprintf(fp, "3 %d %d %d\n",
            gconnect[i][0],
            gconnect[i][1],
            gconnect[i][2] );
    fclose( fp );

    vnorm_filename = (char *)malloc( strlen( filename ) + 2 );
    strcpy( vnorm_filename, filename );
    vnorm_filename[strlen(filename)-4] = 'v';
    vnorm_filename[strlen(filename)-3] = 'n';
    vnorm_filename[strlen(filename)-2] = 'o';
    vnorm_filename[strlen(filename)-1] = 'r';
    vnorm_filename[strlen(filename)+1] = '\0';
    vnorm_filename[strlen(filename)-0] = 'm';
    fv = fopen( vnorm_filename, "w" );
    fprintf(fv, "%d\n", gvertices.count);
    for (i = 0; i < gvertices.count; i++)
        fprintf(fv, "%f  %f  %f\n",
            gvertices.ptr[i].normal.x,
            gvertices.ptr[i].normal.y,
            gvertices.ptr[i].normal.z);
    fclose( fv );

    aoff_filename = (char *)malloc( strlen( filename ) + 1 );
    strcpy( aoff_filename, filename );
    aoff_filename[strlen(filename)-4] = 'a';
    aoff_filename[strlen(filename)-3] = 'o';
    aoff_filename[strlen(filename)-2] = 'f';
    aoff_filename[strlen(filename)-1] = 'f';
    fa = fopen( aoff_filename, "w" );
    fprintf( fa, "# %s + %s + %s + %s\n", xy, xz, yz, c );
    tmp_filename = (char *)malloc( strlen( filename ) + 1 );
    strcpy( tmp_filename, filename );
    tmp_filename += strbpos( tmp_filename, '/' ) + 1;
    fprintf( fa, "geometry\tindexed_poly\tfff\t%s\n", tmp_filename );
    tmp_filename = (char *)malloc( strlen( filename ) + 1 );
    strcpy( tmp_filename, vnorm_filename );
    tmp_filename += strbpos( tmp_filename, '/' ) + 1;
    fprintf( fa, "vertex_normals\tgeneric\t\tfff\t%s\n", tmp_filename );
    fclose( fa );

    dfcloseall();
}
