
#include "unix.h"
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>

unsigned int  num_used_pixels = 0;
unsigned long *used_pixels;

unsigned long AllocNamedColor( color_name )
char *color_name;
{
    XColor color;
    extern Display *display;

    if( XParseColor( display,
        DefaultColormap( display, DefaultScreen( display ) ),
        color_name, &color ) == 0 ) {
        fprintf( stderr, "AllocNamedColor: Could not XParseColor for = %s\n",
            color_name );
    }
    if( XAllocColor( display,
        DefaultColormap( display, DefaultScreen( display ) ), &color ) == 0 ) {
        fprintf( stderr, "AllocNamedColor: Could not XAllocColor for %s\n",
            color_name );
        return;
    }
    return( color.pixel );
}


unsigned long AllocColor( display, colormap, color_name )
Display *display;
Colormap colormap;
char *color_name;
{
    XColor color;

    if( XParseColor( display, colormap, color_name, &color ) == 0 ) {
        fprintf( stderr, "AllocColor: Could not XParseColor for = %s\n",
            color_name );
        return;
    }
    if( XAllocColor( display, colormap, &color ) == 0 ) {
        fprintf( stderr, "AllocColor: Could not XAllocColor for %s\n",
            color_name );
        return;
    }

    if( num_used_pixels == 0 ) {
        if( (used_pixels = (unsigned long *)
            malloc((num_used_pixels+1)*sizeof(unsigned long))) == NULL )
            fprintf( stderr, "FreeColor: Could not malloc for %s\n",
                color_name );
        used_pixels[num_used_pixels++] = color.pixel;
    }
    else {
        if( (used_pixels = (unsigned long *) realloc((char *)used_pixels,
            (num_used_pixels+1)*sizeof(unsigned long))) == NULL )
            fprintf( stderr, "FreeColor: Could not realloc for %s\n",
                color_name );
        used_pixels[num_used_pixels++] = color.pixel;
    }

    return( color.pixel );
}
